########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../ar.4gm/i_cashe.4gs/midlevel.4gl  1.7  Delta: 9/4/90
########################################################################

globals "globals.4gl"

define
   locked_cust char(1)

#######################################################################
# "ok" functions:
#  - the following "ok" functions can be placed here:
#    ok_add(), ok_update(), ok_delete(), ok_find(), ok_browse(), ok_next(),
#    ok_prev(), ok_tab(), ok_options(), ok_exit(), and ok_bang().
#  - their purpose is to allow/disallow that function from the ring menu.
#  - if their return value is true, then the function is allowed, and
#    if it is false, then the function is not allowed.
#  - following is an example of a smart delete function:
#
#    ##################################################################
#    function ok_delete()
#    # returning true or false based upon ok to delete
#    ##################################################################
#    #
#        if posted = "Y"
#        then
#            error " Cannot delete a posted record "
#            return false
#        end if
#        return true
#    end function
#    # ok_delete()
#
#######################################################################


##################################################################
function ok_add()
# returning true or false based upon ok to add
##################################################################
#
    let add_update = "A"
    let curr_cust_code = null
    let locked_cust = "N"
    return true
end function
# ok_add

##################################################################
function ok_update()
# returning true or false based upon ok to update
##################################################################
#
    if m_rcashe.cust_code is null and curr_form = 1
    then
        display form screen2
        let curr_form = 2
        call llh_display()
        call lld_display()
    end if
    if m_rcashe.cust_code is not null and curr_form = 2
    then
        display form screen1
        let curr_form = 1
        call llh_display()
        call lld_display()
    end if
    if m_rcashe.cust_code is not null
    then
        if not cust_lock(m_rcashe.cust_code)
        then
             call fg_err(11) # "Cannot lock customer record."
             return false
        end if
        call please_wait()
        call init_cust_detail(rec1_cnt)
        call lld_display()
    end if
    let add_update = "U"
    let curr_cust_code = p_rcashe.cust_code
    return true
end function
# ok_update

#######################################################################
function mlh_init()
# returning the where clause string
#######################################################################
#  - this functions only purpose is to return a string that will be used
#    for the where clause in the initial document grouping that takes
#    place when the program is loaded.
#  - examples:
#   return "1=1" (select all rows)
#   return "1=2" (select no rows)
#   return "posted = 0" (select only unposted documents)
#   return "rowid = 1" (select only the first row - for control files)
#
    return "1=2"
end function
# mlh_init()

#######################################################################
function mlh_validate()
#######################################################################
#   mid level data validation.  this function is called before any
#   disk writes.  if informix tries to write invalid data, part of the row
#   will be written, and part of it won't.  sql bomb's out.  the data
#   should have been checked in ll_input(), but us programmers can't
#   be perfect all of the time.  instead of data being compromised, the
#   end user doesn't get to write that screen.  it ticks them off, but
#   at least they don't have partial records on disk.
#
    whenever error continue
    call llh_m_prep()
    validate m_rcashe.* like strcashe.*
    whenever error call error_handler
end function
# mlh_validate()

#######################################################################
function mld_validate()
#######################################################################
#   like above, only validation must occur for each line
#
    define
        n   smallint    # working number
    whenever error continue
    for n = 1 to rec1_cnt
        call lld_m_prep(n)
        validate m_rcashd.* like strcashd.*
    end for
    whenever error call error_handler
end function
# mld_validate

#######################################################################
function mlh_clear()
#######################################################################
#
    initialize p_rcashe.* to null
    initialize r_rcashe.* to null
    initialize m_rcashe.* to null
end function
# mlh_clear

#######################################################################
function mld_clear()
#######################################################################
#
    define
        n       smallint            # working number
    initialize m_rcashd.* to null    # clear the m_record
    # clear all p_ records in the array
    for n = 1 to rec1_max
        initialize p_rcashd[n].* to null   # clear the p_record
        initialize q_rcashd[n].* to null   # clear the q_record
        initialize r_rcashd[n].* to null   # clear the r_record
    end for
    let rec1_cnt = 0                # reset the number of active elements
end function
# mld_mclear()

#######################################################################
function mlh_construct()
# returning criteria
#######################################################################
#
    define
        criteria    char(256)   # selection criteria holding bin
    # construct the search criteria from the screen
    if curr_form = 1
    then
        construct criteria
            on
                strcashe.rcpt_date,
                strcashe.cust_code,
                strcashe.check_no,
                strcashe.doc_desc
            from
                s_rcashe.rcpt_date,
                s_rcashe.cust_code,
                s_rcashe.check_no,
                s_rcashe.doc_desc
    end if
    if curr_form = 2
    then
        construct criteria
            on
                strcashe.rcpt_date,
                strcashe.check_no,
                strcashe.doc_desc
            from
                t_rcashe.rcpt_date,
                t_rcashe.check_no,
                t_rcashe.doc_desc
    end if
    return criteria
end function
# mlh_construct

#######################################################################
function mlh_define_cur(criteria)
#######################################################################
#
    define
        criteria    char(256),  # selection criteria holding bin
        find_specs  char(512),  # select sql specification
        curs_count  integer     # number of elements in the cursor

    if criteria = "1=2" then let sqlca.sqlerrd[3] = 0 return end if

    # get the count into curs_count
    let find_specs =
        "select count(*) from strcashe where ",
        criteria clipped
    prepare get_count from find_specs
    declare count_cursor cursor for get_count
    open count_cursor
    fetch count_cursor into curs_count
    close count_cursor
    # define the cursor
    let find_specs =
        "select rowid, doc_no",
        " from strcashe where ",criteria clipped,
        " order by doc_no"
    prepare get_stmt from find_specs
    declare ref_cursor scroll cursor for get_stmt
    open ref_cursor
    let sqlca.sqlerrd[3] = curs_count
    return
end function
#mlh_define_cur()


#######################################################################
function mlh_fetch_cur(curno)
# returning rowid in sqlca.sqlerrd[6]
#######################################################################
#
    define
        curno       integer,    # absolute cursor number
        curs_rowid integer, p1 char(1)

    if curno = 0 then let sqlca.sqlerrd[6] = -1  return end if

    fetch absolute curno ref_cursor into
            curs_rowid, p1
    let sqlca.sqlerrd[6] = curs_rowid
    return
end function
# mlh_fetch_cur()

#######################################################################
function mlh_close_cur()
#######################################################################
#
    close ref_cursor
end function
# mlh_close_cur()

#######################################################################
function mlh_lock(ring_rowid)
#######################################################################
#   - this function opens a cursor for update selecting just the
#     record defined by rowid.
#   - because it fetches the record with an update cursor, the record
#     is locked.
#   - it's only meaning in life is to lock the record. not read it.
#
    define
        ring_rowid  integer,    # rowid of record to read
        getid       integer     # something to fetch into
    whenever error continue
    declare lock_cursor cursor for
        select rowid into getid from strcashe where rowid = ring_rowid
        for update
    open lock_cursor
    fetch lock_cursor
    whenever error call error_handler
    return
end function
# mlh_lock()

#######################################################################
function mlh_unlock()
#######################################################################
#
    close lock_cursor
end function
# mlh_unlock()

#######################################################################
function mld_scroll()
#######################################################################
#   - this function calls display array for the window.
#
    if rec1_cnt < 1
    then
        return
    end if
    call set_count(rec1_cnt)
    if curr_form = 1
    then
        display array q_rcashd to s_rcashd.*
            on key (control-i)
                exit display
        end display
    end if
    if curr_form = 2
    then
        display array r_rcashd to t_rcashd.*
            on key (control-i)
                exit display
        end display
    end if
end function
# mld_scroll

##########################################################################
function mld_arr_count()
##########################################################################
#   - this function sets the rec1_cnt variable instead of informix's
#     arr_count() function that can't be trusted.
#
    define
        n       smallint        # working number
    for n = rec1_max to 1 step -1
        if      p_rcashd[n].pay_method  is not null
            or  p_rcashd[n].inv_no  is not null
            or  p_rcashd[n].due_date  is not null
            or  p_rcashd[n].disc_date  is not null
            or  p_rcashd[n].disc_bal  is not null
            or  p_rcashd[n].balance  is not null
            or  p_rcashd[n].disc_amt  is not null
            or  p_rcashd[n].dist_acct  is not null
            or  p_rcashd[n].dist_department  is not null
            or  p_rcashd[n].acct_desc  is not null
            or  p_rcashd[n].dist_amt  is not null
            or  p_rcashd[n].dist_deb_cred  is not null
        then
          if n <= arr_count()
          then
            let rec1_cnt = n
            return
          else
            let rec1_cnt = arr_count()
            initialize p_rcashd[rec1_cnt+1].* to null   # clear the p_record
            initialize q_rcashd[rec1_cnt+1].* to null   # clear the q_record
            initialize r_rcashd[rec1_cnt+1].* to null   # clear the r_record
            return
          end if
        end if
    end for
    let rec1_cnt = 0
    return
end function
# mld_arr_count()

#######################################################################
function cust_lock(p_cust_code)
#######################################################################
#   - this function opens a cursor for update
#   - because it fetches the cusotmer record with an update cursor, 
#     the customer record is locked.
#   - it's only meaning in life is to lock the record. not read it.
#
    define
        p_cust_code like strcustr.cust_code,    # customer code to read
        getid       integer                     # something to fetch into

    declare cust_lock cursor for
        select rowid into getid from strcustr 
             where cust_code = p_cust_code for update
    open cust_lock

    whenever error continue
    fetch cust_lock
    whenever error call error_handler

    if sqlca.sqlcode != 0 then return false end if

    let locked_cust = "Y"

    return true
end function
# cust_lock()

#######################################################################
function cust_unlock()
#######################################################################
#
    if locked_cust = "Y"
    then
        close cust_lock
    end if
end function
# cust_unlock()
