##############################################################################
# Copyright 1988 Fourgen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../ar.4gm/i_shipr.4gs/midlevel.4gl  1.2  Last Delta: 10/20/89
##############################################################################

globals "globals.4gl"

#######################################################################
# "ok" functions:
#  - the following "ok" functions can be placed here:
#    ok_add(), ok_update(), ok_delete(), ok_find(), ok_browse(), ok_next(),
#    ok_prev(), ok_tab(), ok_options(), ok_exit(), and ok_bang().
#  - their purpose is to allow/disallow that function from the ring menu.
#  - if their return value is true, then the function is allowed, and
#    if it is false, then the function is not allowed.
#  - following is an example of a smart delete function:
#
#    ##################################################################
#    function ok_delete()
#    # returning true or false based upon ok to delete
#    ##################################################################
#    #
#        if posted = "Y"
#        then
#            error " Cannot delete a posted record "
#            return false
#        end if
#        return true
#    end function
#    # ok_delete()
#
#######################################################################


##################################################################
function ok_add()
# returning true or false based upon ok to add
##################################################################
#
    let orig_cust_code = "null"
    let orig_ship_to_code = "null"
    return true
end function
# ok_add()

##################################################################
function ok_update()
# returning true or false based upon ok to update
##################################################################
#
    let orig_cust_code = p_rshipr.cust_code
    let orig_ship_to_code = p_rshipr.ship_to_code
    return true
end function
# ok_update()

#######################################################################
function mlh_init()
# returning the where clause string
#######################################################################
#  - this functions only purpose is to return a string that will be used
#    for the where clause in the initial document grouping that takes
#    place when the program is loaded.
#  - examples:
#   return "1=1" (select all rows)
#   return "1=2" (select no rows)
#   return "posted = 0" (select only unposted documents)
#   return "rowid = 1" (select only the first row - for control files)
#
    return "1=2"
end function
# mlh_init()

#######################################################################
function mlh_validate()
#######################################################################
#   mid level data validation.  this function is called before any
#   disk writes.  if informix tries to write invalid data, part of the row
#   will be written, and part of it won't.  sql bomb's out.  the data
#   should have been checked in ll_input(), but us programmers can't
#   be perfect all of the time.  instead of data being compromised, the
#   end user doesn't get to write that screen.  it ticks them off, but
#   at least they don't have partial records on disk.
#
    whenever error continue
    call llh_m_prep()
    validate m_rshipr.* like strshipr.*
    whenever error call error_handler
end function
# mlh_validate()

#######################################################################
function mlh_clear()
#######################################################################
#
    initialize p_rshipr.* to null
    initialize m_rshipr.* to null
end function
# mlh_clear

#######################################################################
function mlh_construct()
# returning criteria
#######################################################################
#
    define
        criteria    char(256)   # selection criteria holding bin
    # construct the search criteria from the screen
    construct criteria
        on
            strshipr.cust_code,
            strshipr.ship_to_code,
            strshipr.bus_name,
            strshipr.taxable,
            strshipr.contact,
            strshipr.phone,
            strshipr.address1,
            strshipr.address2,
            strshipr.city,
            strshipr.state,
            strshipr.zip,
            strshipr.country,
            strshipr.spec_shipping
        from
            s_rshipr.cust_code,
            s_rshipr.ship_to_code,
            s_rshipr.bus_name,
            s_rshipr.taxable,
            s_rshipr.contact,
            s_rshipr.phone,
            s_rshipr.address1,
            s_rshipr.address2,
            s_rshipr.city,
            s_rshipr.state,
            s_rshipr.zip,
            s_rshipr.country,
            s_rshipr.spec_shipping
    return criteria
end function
# mlh_construct

#######################################################################
function mlh_define_cur(criteria)
#######################################################################
#
    define
        criteria    char(256),  # selection criteria holding bin
        find_specs  char(512),  # select sql specification
        curs_count  integer     # number of elements in the cursor

    if criteria = "1=2" then let sqlca.sqlerrd[3] = 0 return end if

    # get the count into curs_count
    let find_specs =
        "select count(*) from strshipr where ",
        criteria clipped
    prepare get_count from find_specs
    declare count_cursor cursor for get_count
    open count_cursor
    fetch count_cursor into curs_count
    close count_cursor
    # define the cursor
    let find_specs =
        "select rowid, cust_code, ship_to_code",
        " from strshipr where ",criteria clipped,
        " order by cust_code, ship_to_code"
    prepare get_stmt from find_specs
    declare ref_cursor scroll cursor for get_stmt
    open ref_cursor
    let sqlca.sqlerrd[3] = curs_count
    return
end function
#mlh_define_cur()


#######################################################################
function mlh_fetch_cur(curno)
# returning rowid in sqlca.sqlerrd[6]
#######################################################################
#
    define
        curno       integer,    # absolute cursor number
        curs_rowid integer, p1 char(1), p2 char(1)

    if curno = 0 then let sqlca.sqlerrd[6] = -1  return end if

    fetch absolute curno ref_cursor into
            curs_rowid, p1, p2
    let sqlca.sqlerrd[6] = curs_rowid
    return
end function
# mlh_fetch_cur()

#######################################################################
function mlh_close_cur()
#######################################################################
#
    close ref_cursor
end function
# mlh_close_cur()

#######################################################################
function mlh_lock(ring_rowid)
#######################################################################
#   - this function opens a cursor for update selecting just the
#     record defined by rowid.
#   - because it fetches the record with an update cursor, the record
#     is locked.
#   - it's only meaning in life is to lock the record. not read it.
#
    define
        ring_rowid  integer,    # rowid of record to read
        getid       integer     # something to fetch into
    whenever error continue
    declare lock_cursor cursor for
        select rowid into getid from strshipr where rowid = ring_rowid
        for update
    open lock_cursor
    fetch lock_cursor
    whenever error call error_handler
    return
end function
# mlh_lock()

#######################################################################
function mlh_unlock()
#######################################################################
#
    close lock_cursor
end function
# mlh_unlock()

