##############################################################################
# Copyright 1988 Fourgen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../ar.4gm/i_termr.4gs/header.4gl  1.4  Last Delta: 8/31/90
##############################################################################

globals "globals.4gl"

#######################################################################
function llh_input()
# returning -1 if tab pressed (next window), 0 otherwise
#######################################################################
#
    define
        old_terms_code      like strtermr.terms_code

    input p_rtermr.* without defaults from s_rtermr.*
        on key (control-e)
            call fg_err(0)
        on key (control-w)
            call fg_help(1)
        on key (control-f)
            # add/update user defined fields...
            if m_rtermr.terms_code is null
              then call al_addlz("strtermr",p_rtermr.terms_code,"","")
              else call al_addlz("strtermr",m_rtermr.terms_code,"","")
            end if
        before field terms_code
            let old_terms_code = p_rtermr.terms_code
        after field terms_code
            if (p_rtermr.terms_code is not null and
               p_rtermr.terms_code != old_terms_code) or
               (p_rtermr.terms_code is not null and old_terms_code is null)
            then
                if ar_termd(p_rtermr.terms_code)
                then
                    call fg_err(1) # duplicate terms code found
                    let p_rtermr.terms_code = old_terms_code
                    display by name p_rtermr.terms_code
                    next field terms_code
                end if
            end if

    end input
    return 0
end function
# llh_input()

#######################################################################
function llh_chkinput()
#   returns true/false based upon good/bad data in screen
#######################################################################
#   this function is called upon exiting the input statement.
#   the "after input" clause in the input statement can't call
#   this because a tab (control-i) needs to run this function also.
#
#   if you have any must-fill type fields on the screen, you can use
#   the function "mustfill()" that comes with infourgen.  it's syntax is:
#       mustfill(variable_name,"description of that variable")
#           returning true/false based upon no_data/data in that field.
#   it's use in this context is:
#       if mustfill(p_gchrt.acct_no,"Account Number") then return false end if
#
        if mustfill(p_rtermr.terms_code,"Terms Code")
        then
            return false
        end if
        if mustfill(p_rtermr.terms_desc,"Description of Terms")
        then
            return false
        end if
        if mustfill(p_rtermr.due_days,"Number of Due Days")
        then
            return false
        end if
        if p_rtermr.disc_days is null
        then
            let p_rtermr.disc_days = 0
            display by name p_rtermr.disc_days
        end if
        if p_rtermr.disc_pct is null
        then
            let p_rtermr.disc_pct = 0
            display by name p_rtermr.disc_pct
        end if
    return true
end function
# llh_chkinput()

#######################################################################
function llh_display()
#######################################################################
#   - display all p_* variables onto the screen
#
    display p_rtermr.* to s_rtermr.*
end function
# llh_display()

#######################################################################
function llh_p_prep()
#######################################################################
#   - this function creates the p_* record from the m_* record
#
    let p_rtermr.terms_code = m_rtermr.terms_code
    let p_rtermr.terms_desc = m_rtermr.terms_desc
    let p_rtermr.due_days = m_rtermr.due_days
    let p_rtermr.disc_days = m_rtermr.disc_days
    let p_rtermr.disc_pct = m_rtermr.disc_pct
end function
# llh_p_prep()

#######################################################################
function llh_m_prep()
#######################################################################
#   - this function creates the m_* record from the p_* record
#
    # change key to user defined fields
    # if key was changed during input
    if   m_rtermr.terms_code is not null
     and m_rtermr.terms_code != p_rtermr.terms_code
    then
        call al_addlnk("strtermr",m_rtermr.terms_code,p_rtermr.terms_code)
    end if
    let m_rtermr.terms_code = p_rtermr.terms_code
    let m_rtermr.terms_desc = p_rtermr.terms_desc
    let m_rtermr.due_days = p_rtermr.due_days
    let m_rtermr.disc_days = p_rtermr.disc_days
    let m_rtermr.disc_pct = p_rtermr.disc_pct
end function
# llh_m_prep()

#######################################################################
function llh_read(ring_rowid)
#######################################################################
#   - this function reads in all of the p_* variables from disk.
#   - it does not use a cursor, so it doesn't lock the row.
#
    define
        ring_rowid  integer     # rowid of record to read
    whenever error continue
    select * into m_rtermr.* from strtermr where rowid = ring_rowid
    whenever error call error_handler
    if sqlca.sqlcode != 0
    then
        return
    end if
    call llh_p_prep()
    return
end function
# llh_read()

#######################################################################
function llh_delete(ring_rowid)
#######################################################################
#
    define
        ring_rowid  integer     # rowid passed
    delete from strtermr
        where rowid = ring_rowid
    if sqlca.sqlcode = 0
    then
        # delete user defined fields
        call al_addld("strtermr",p_rtermr.terms_code)
    end if
end function
# llh_delete()

#######################################################################
function llh_add()
#######################################################################
#   - insert a new reference record, and all n-lines associated with it.
#
    whenever error continue
    insert into strtermr values(m_rtermr.*)
    whenever error call error_handler
end function
# llh_add()

#######################################################################
function llh_update(ring_rowid)
#######################################################################
#
    define
        ring_rowid  integer     # rowid passed
    update strtermr set strtermr.* = m_rtermr.*
        where rowid = ring_rowid
end function
# llh_update()

