########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../ar.4gm/p_invpst.4gs/lowlevel.4gl  1.13  Delta: 8/31/90
########################################################################

globals "globals.4gl"

#######################################################################
function before_group(group_key)
#######################################################################
#
    define
        file_type char(2),      # IN/CM/DM
        inv_doc_no integer,     # invoice doc_no (or apply-to invoice doc_no)
        group_key   char(20)    # group identification

    if group_key = "first_row" then
        if check_post = "POST"
        then
            begin work
                call auto_next("strcntrc","ar_post_no")
                    returning post_no
            commit work 
            let rpt.post_seq = post_no using "Posting Sequence: <<<<<<"
        else
            let rpt.post_seq = ""
        end if
        return
    end if

    if group_key = "doc_no" then
    # For each document do the following:
    # - read strinvce.* into p_rinvce (wouldn't all fit into cursor)
    # - begin work & set some accumulators to 0
    # - if post, then grab the next doc_no, if not, set doc_no to ""
    # - call ar_post
    # - set the appropriate rpt.* variables
    # - delete strinvce unless it's marked for recurring
    # - set posted to "Y" in strinvce (if not deleted)
    
        let rpt.problm0 = ""
        let new_doc_no = curs.doc_no
        whenever error continue
        select * into p_rinvce.* from strinvce where doc_no = curs.doc_no
        whenever error call error_handler
        if sqlca.sqlcode != 0
        then
            if check_post = "POST"
            then
                call fg_err(1) # Cannot read strinvce table
            end if
            call end_report("An SQL error has occured while reading strinvce.")
        end if
        if check_post = "POST"
        then
            begin work
            if curs.ok_to_post != "Y"
            then
                rollback work
                call fg_err(4) # edit list hasn't been run on this document
                call end_report("The edit list hasn't been run on this document.")
            end if
            call auto_next("strcntrc","ar_doc_no")
                returning new_doc_no
            let rpt.txt_doc_no = new_doc_no using "<<<<<<<<<"
            let group_key = curs.doc_no using "<<<<<<<<<<", "-UNPOSTED"
            call al_addlnk("stxtranrAR",group_key,new_doc_no)
        else
            let rpt.txt_doc_no = "UNASSIGNED"
        end if
        let rpt.doc_no = curs.doc_no
        if p_rinvce.ar_debit_credit = "DB"
        then
            let ar_signed_amt = p_rinvce.ar_amount
            let disc_signed_amt = p_rinvce.disc_amount
        else
            let ar_signed_amt = p_rinvce.ar_amount * -1
            let disc_signed_amt = p_rinvce.disc_amount * -1
        end if

        if curs.file_type = "I" or curs.file_type = "F"
        then
            if curs.file_type = "I"
            then
              let rpt.inv_type = "INVOICE"
              let file_type = "IN"
            else
              let rpt.inv_type = "FINANCE CHARGE"
              let file_type = "FC"
            end if
            let rpt.ref_code = ""
            let rpt.ref_verb = ""
            let inv_doc_no = new_doc_no
        else
            if curs.file_type = "C"
            then
                let rpt.inv_type = "CREDIT MEMO"
                let file_type = "CM"
            else
                let rpt.inv_type = "DEBIT MEMO"
                let file_type = "DM"
            end if
            let rpt.ref_verb = "Reference:"
            if curs.ref_no is null
            then
                let rpt.ref_code = "ON ACCOUNT"
                let inv_doc_no = 0
            else
                let rpt.ref_code = curs.ref_no using "<<<<<<<<<<"
                let inv_doc_no = curs.ref_no
            end if
        end if # invoice / memo

        # post this guy to a/r...
        if ar_post(
            check_post, "AR", new_doc_no, post_no, today, p_rinvce.inv_date,
            p_rinvce.cust_code, p_rinvce.inv_desc, p_rinvce.inv_no,
            file_type, "A", inv_doc_no, ar_signed_amt, p_rinvce.disc_date,
            disc_signed_amt, p_rinvce.due_date, p_rinvce.ar_acct_no,
            p_rinvce.ar_department, p_rinvce.po_no, p_rinvce.po_date) = false
        then
            if check_post = "POST"
            then
                rollback work
                call fg_error("lib_all","ar_post",post_ar.status)
            end if
            call end_report(post_ar.description)
        end if

        # cause fatal error if a/r isn't installed (very unlikely)
        if post_ar.status = 1
        then
            rollback work
            call fg_error("lib_all","ar_post",post_ar.status)
            let check_post = "POST"  # make end_report fatal...
            call end_report(post_ar.description)
        end if

        # mark it ok for posting if it's the edit list
        if check_post = "CHECK"
        then
            whenever error continue
            update strinvce set ok_to_post = "Y"
                where doc_no = curs.doc_no
            whenever error call error_handler
            if sqlca.sqlcode != 0
            then
                call fg_err(5) # SQL Error: cannot update strinvce
                call end_report("SQL Error: cannot update strinvce")
            end if
            return
        end if

        # either delete the entry, or mark it as posted (only if recurring)
        if p_rinvce.recurring is not null
        then
            whenever error continue
            update strinvce set (posted, recurr_ref) = ("Y", rpt.txt_doc_no)
                where doc_no = curs.doc_no
            whenever error call error_handler
            if sqlca.sqlcode != 0
            then
                rollback work
                call fg_err(2) #An SQL error has occured while posting to strinvce
                call end_report("An SQL error has occured while posting the previous document.")
            end if
        else
            whenever error continue
            delete from strinvce 
                where doc_no = curs.doc_no
            if sqlca.sqlcode = 0
            then
                delete from strinvcd 
                    where doc_no = curs.doc_no
            end if
            whenever error call error_handler
            if sqlca.sqlcode != 0
            then
                rollback work
                call fg_err(3) #An SQL error has occured while deleting strinvce
                call end_report("An SQL error has occured while deleting the previous document.")
            end if
        end if
        return
    end if

end function
# before_group()

#######################################################################
function on_every_row()
#######################################################################
#  this function prepares the report record from the cursor record
#  and other data.
#
    let rpt.description = null
    let rpt.file_type = curs.file_type
    let rpt.debit_credit = curs.debit_credit
    let rpt.line_no = curs.line_no
    let rpt.due_date = curs.due_date
    let rpt.disc_date = curs.disc_date
    let rpt.po_no = curs.po_no
    let rpt.po_date = curs.po_date
    let rpt.inv_date = curs.inv_date
    let rpt.cust_code = curs.cust_code
    let rpt.bus_name = curs.bus_name
    let rpt.inv_no = curs.inv_no
    let rpt.inv_desc = curs.inv_desc
    let rpt.acct_no = curs.acct_no
    let rpt.department = curs.department
    let rpt.amount = curs.amount
    let rpt.debit_credit = curs.debit_credit

    # post to g/l...
    if rpt.acct_no is not null
    then
      if gl_post(
        check_post, "AR", new_doc_no, post_no, today, p_rinvce.inv_date,
        p_rinvce.cust_code, p_rinvce.inv_desc, p_rinvce.inv_no, curs.acct_no,
        curs.department, curs.amount, curs.debit_credit) = false
      then
        if check_post = "POST"
        then
            rollback work
            call fg_error("lib_all","gl_post",post_gl.status)
        end if
        call end_report(post_gl.description)
      end if
      # account name is stored in post_gl.acct_desc
      let rpt.description = post_gl.acct_desc
    end if

    return
end function
# on_every_row()

#######################################################################
function after_group(group_key)
#######################################################################
#
    define
        group_key   char(20)    # group identification

    if group_key = "doc_no" then
        call last_few() # print the last few distributions
        if gl_last() = false
        then
            if check_post = "POST"
            then
                rollback work
                call fg_error("lib_all","gl_last",post_gl.status)
            end if
            call end_report(post_gl.description)
        end if

        if ar_last(check_post, "IN", ar_signed_amt, curs.cust_code) = false
        then
            if check_post = "POST"
            then
                rollback work
                call fg_error("lib_all","ar_last",post_ar.status)
            end if
            call end_report(post_ar.description)
        end if
        if check_post = "POST"
        then
            commit work
        end if
        return
    end if

    if group_key = "line_no" then
        # insert "after group of line_no" code here
        return
    end if

    if group_key = "last_row" then
        if rpt.problm1 is null
        then
            let rpt.problm1 =
                "**** End of Report.  No errors were detected."
        end if
        return
    end if

end function
# after_group()

#######################################################################
function last_few()
#######################################################################
#  this function posts the last few lines of the invoice to distd
#  and to the report.
#  (last few lines correspond to the tax, freight, misc, and ar accounts)
#
    if curs.line_no is null
    then
        let curs.line_no = 0
    end if

    if p_rinvce.tax_amount != 0
    then
        call ml_output()       # output the current row
        let curs.line_no = curs.line_no + 1
        let curs.acct_no = p_rinvce.tax_acct_no
        let curs.department = p_rinvce.tax_department
        let curs.amount = p_rinvce.tax_amount
        let curs.debit_credit = p_rinvce.tax_debit_credit
        call on_every_row()    # prepare new row data
    end if
    
    if p_rinvce.frght_amount != 0
    then
        call ml_output()       # output the current row
        let curs.line_no = curs.line_no + 1
        let curs.acct_no = p_rinvce.frght_acct_no
        let curs.department = p_rinvce.frght_department
        let curs.amount = p_rinvce.frght_amount
        let curs.debit_credit = p_rinvce.frght_debit_credit
        call on_every_row()    # prepare new row data
    end if
    
    if p_rinvce.misc_amount != 0
    then
        call ml_output()       # output the current row
        let curs.line_no = curs.line_no + 1
        let curs.acct_no = p_rinvce.misc_acct_no
        let curs.department = p_rinvce.misc_department
        let curs.amount = p_rinvce.misc_amount
        let curs.debit_credit = p_rinvce.misc_debit_credit
        call on_every_row()    # prepare new row data
    end if
    
    if p_rinvce.ar_amount != 0
    then
        call ml_output()       # output the current row
        let curs.line_no = curs.line_no + 1
        let curs.acct_no = p_rinvce.ar_acct_no
        let curs.department = p_rinvce.ar_department
        let curs.amount = p_rinvce.ar_amount
        let curs.debit_credit = p_rinvce.ar_debit_credit
        call on_every_row()    # prepare new row data
    end if
    
end function
# last_few()

#######################################################################
function end_report(why)
#######################################################################
#  this function calls the last line of the report for abnormal termination.
#
    define
        why char(80)

    if check_post = "POST"
    then
        let rpt.problm1 =
            "***** The posting has terminated for the following reason:"
        let rpt.problm2 = "***** ", why clipped
        let rpt.problm3 =
            "***** It HAS NOT been posted.  Documents prior to it HAVE been posted."
        call ml_output()
        finish report report1
        exit program(1)
    else
        let rpt.problm0 = "***** ",why clipped
        let rpt.problm1 = "***** THIS REPORT HAS ERRORS DETECTED"
    end if

end function
# end_report()
